@@ -1,3 +1,5 @@
+latest_standard = 2020
+
 default_client_config  = {
         'exception_on_negative_response'	: True,	
         'exception_on_invalid_response'		: True,
@@ -9,9 +11,11 @@ default_client_config  = {
         'dtc_snapshot_did_size' 	: 2,		# Not specified in standard. 2 bytes matches other services format.
         'server_address_format'		: None,		# 8,16,24,32,40
         'server_memorysize_format'	: None,		# 8,16,24,32,40
-        'data_identifiers' 			: {},
-        'input_output' 				: {},
-        'request_timeout'			: 5,
-        'p2_timeout'				: 1, 
-        'p2_star_timeout'			: 5,
+        'data_identifiers' 		: {},
+        'input_output' 			: {},
+        'request_timeout'		: 5,
+        'p2_timeout'			: 1, 
+        'p2_star_timeout'		: 5,
+        'standard_version'              : latest_standard,  # 2006, 2013, 2020
+        'use_server_timing'             : True
 }
