@@ -20,6 +20,15 @@ try:
 except Exception as e:
     _import_isotp_err = e
 
+try:
+    from udsoncan.j2534 import J2534
+    from udsoncan.j2534 import Protocol_ID
+    _import_j2534_err = None
+except Exception as e:
+    _import_j2534_err = e
+
+
+
 from udsoncan.Request import Request
 from udsoncan.Response import Response
 from udsoncan.exceptions import TimeoutException
@@ -520,3 +529,185 @@ class PythonIsoTpConnection(BaseConnection):
             except Exception as e:
                 self.exit_requested = True
                 self.logger.error(str(e))
+
+class J2534Connection(BaseConnection):
+    """
+    Sends and receives data through a J2534 Interface. 
+    A windows DLL and a J2534 interface must be installed in order to use this connection
+
+    :param windll: The path to the windows DLL for the J2534 interface (example: 'C:/Program Files{x86}../../openport 2.0/op20pt32.dll')
+    :type interface: string
+    :param rxid: The reception CAN id
+    :type rxid: int 
+    :param txid: The transmission CAN id
+    :type txid: int
+    :param name: This name is included in the logger name so that its output can be redirected. The logger name will be ``Connection[<name>]``
+    :type name: string
+    :param debug: This will enable windows debugging mode in the dll (see tactrix doc for additional information)
+    :type debug: boolean
+    :param args: Optional parameters list (Unused right now).
+    :type args: list
+    :param kwargs: Optional parameters dictionary Unused right now).
+    :type kwargs: dict
+
+    """
+    def __init__(self, windll, rxid, txid, name=None, debug = False, *args, **kwargs):
+
+        BaseConnection.__init__(self, name)
+
+        #Set up a J2534 interface using the DLL provided
+        self.interface = J2534(windll = windll, rxid = rxid, txid = txid)
+
+        #Set the protocol to ISO15765, Baud rate to 500000
+        self.protocol = Protocol_ID.ISO15765
+        self.baudrate = 500000
+
+        #Open the interface (connect to the DLL)
+        result, self.devID = self.interface.PassThruOpen()
+
+        if debug:
+            result = self.interface.PassThruIoctl(Handle = 0,IoctlID = Ioctl_Flags.TX_IOCTL_SET_DLL_DEBUG_FLAGS, ioctlInput = Ioctl_Flags.TX_IOCTL_DLL_DEBUG_FLAG_J2534_CALLS)
+
+        #Get the firmeware and DLL version etc, mainly for debugging output
+        self.result, self.firmwareVersion, self.dllVersion, self.apiVersion = self.interface.PassThruReadVersion(self.devID)
+        self.logger.info("J2534 FirmwareVersion: " + str(self.firmwareVersion.value) + ", dllVersoin: " + str(self.dllVersion.value) + ", apiVersion" + str(self.apiVersion.value))
+
+        #get the channel ID of the interface (used for subsequent communication)
+        self.result, self.channelID = self.interface.PassThruConnect(self.devID, self.protocol.value, self.baudrate)
+
+        #Set the filters and clear the read buffer (filters will be set based on tx/rxids)
+        self.result = self.interface.PassThruStartMsgFilter(self.channelID, self.protocol.value)
+        self.result = self.interface.PassThruIoctl(self.channelID, Ioctl_ID.CLEAR_RX_BUFFER)
+
+
+        self.rxqueue = queue.Queue()
+        self.exit_requested = False
+        self.opened = False
+
+    def open(self):
+        self.exit_requested = False
+        self.rxthread = threading.Thread(target=self.rxthread_task)
+        self.rxthread.daemon = True
+        self.rxthread.start()
+        self.opened = True
+        self.logger.info('J2534 Connection opened')
+        return self
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
+    def is_open(self):
+        return self.opened
+
+    def rxthread_task(self):
+        
+        while not self.exit_requested:
+            
+            try:
+                result, data, numMessages = self.interface.PassThruReadMsgs(self.channelID, self.protocol.value, 1, 1)
+                
+                if data is not None:
+                    self.rxqueue.put(data)
+            except Exception:
+                self.logger.critical("Exiting J2534 rx thread")
+                self.exit_requested = True
+
+
+    def close(self):
+        self.exit_requested = True
+        self.rxthread.join()
+        result = self.interface.PassThruDisconnect(self.channelID)
+        self.opened = False
+        self.logger.info('J2534 Connection closed')
+
+    def specific_send(self, payload):
+        result = self.interface.PassThruWriteMsgs(self.channelID, payload, self.protocol.value)
+
+    def specific_wait_frame(self, timeout=4):
+        if not self.opened:
+            raise RuntimeError("J2534 Connection is not open")
+
+        timedout = False
+        frame = None
+        try:
+            frame = self.rxqueue.get(block=True, timeout=timeout)
+
+        except queue.Empty:
+            timedout = True
+
+        if timedout:
+            raise TimeoutException("Did not received response from J2534 RxQueue (timeout=%s sec)" % timeout)
+
+        return frame
+
+    def empty_rxqueue(self):
+        while not self.rxqueue.empty():
+            self.rxqueue.get()
+
+class FakeConnection(BaseConnection):
+    """
+    Sends and receives STATIC data defined in a local dict. 
+    Used so that an application can be tested without a live can network
+    """
+    def __init__(self, name=None, debug = False, *args, **kwargs):
+
+        BaseConnection.__init__(self, name)
+
+        self.rxqueue = queue.Queue()
+        
+        self.exit_requested = False
+        self.opened = False
+
+        self.ResponseData = {b'\x10\x03': b'\x50\x03\x12\x23\x34\x45',
+                b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3': b'\x22\xf1\x90\xf1\x89\xf1\x91\xf8\x06\xf1\xa3'}
+
+
+    def open(self):
+        self.opened = True
+        self.logger.info('Fake Connection opened')
+        return self
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
+    def is_open(self):
+        return self.opened
+
+    def close(self):
+        self.exit_requested = True
+        self.opened = False
+        self.logger.info('Fake Connection closed')
+
+    def specific_send(self, payload):
+        self.rxqueue.put(self.ResponseData[payload])
+
+    def specific_wait_frame(self, timeout=4):
+        if not self.opened:
+            raise RuntimeError("Fake Connection is not open")
+
+        timedout = False
+        frame = None
+        try:
+            frame = self.rxqueue.get(block=True, timeout=timeout)
+            #frame = self.rxqueue.get(block=True, timeout=5)
+
+        except queue.Empty:
+            timedout = True
+
+        if timedout:
+            raise TimeoutException("Did not received response from J2534 RxQueue (timeout=%s sec)" % timeout)
+
+        return frame
+
+    def empty_rxqueue(self):
+        while not self.rxqueue.empty():
+            self.rxqueue.get()
+
+    
+
