@@ -1,6 +1,7 @@
 from . import *
 from udsoncan.Response import Response
 from udsoncan.exceptions import *
+from udsoncan.configs import latest_standard
 import struct
 
 class ClearDiagnosticInformation(BaseService):
@@ -14,7 +15,7 @@ class ClearDiagnosticInformation(BaseService):
                                                     ]
 
     @classmethod
-    def make_request(cls, group=0xFFFFFF):
+    def make_request(cls, group=0xFFFFFF, memory_selection=None, standard_version = latest_standard):
         """
         Generates a request for ClearDiagnosticInformation
 
@@ -30,6 +31,14 @@ class ClearDiagnosticInformation(BaseService):
         mb = (group >> 8) & 0xFF
         lb = (group >> 0) & 0xFF 
         request.data = struct.pack("BBB", hb,mb,lb)
+
+        # Introduced in ISO-14229-1:2020
+        if memory_selection is not None:
+            if standard_version < 2020:
+                raise NotImplementedError('ClearDiagnosticInformation with Memory Selection is only possible with 2020 version of the standard or above.')
+
+            ServiceHelper.validate_int(memory_selection, min=0, max=0xFF, name='Memory Selection')
+            request.data += struct.pack("B", memory_selection)
         return request
 
     @classmethod
